==> jsclass-3.0.2.tgz: package/src/dom.js <==
JS.DOM = {
  ELEMENT_NODE:                   1,
  ATTRIBUTE_NODE:                 2,
  TEXT_NODE:                      3,
  CDATA_SECTION_NODE:             4,
  ENTITY_REFERENCE_NODE:          5,
  ENTITY_NODE:                    6,
  PROCESSING_INSTRUCTION_NODE:    7,
  COMMENT_NODE:                   8,
  DOCUMENT_NODE:                  9,
  DOCUMENT_TYPE_NODE:             10,
  DOCUMENT_FRAGMENT_NODE:         11,
  NOTATION_NODE:                  12,
  
  ENV: this,
  
  toggleClass: function(node, className) {
    if (this.hasClass(node, className)) this.removeClass(node, className);
    else this.addClass(node, className);
  },
  
  hasClass: function(node, className) {
    var classes = node.className.split(/\s+/);
    return JS.indexOf(classes, className) >= 0;
  },
  
  addClass: function(node, className) {
    if (this.hasClass(node, className)) return;
    node.className = node.className + ' ' + className;
  },
  
  removeClass: function(node, className) {
    var pattern = new RegExp('\\b' + className + '\\b\\s*', 'g');
    node.className = node.className.replace(pattern, '');
  }
};


JS.DOM.Builder = new JS.Class('DOM.Builder', {
  extend: {
    addElement: function(name) {
      this.define(name, function() {
        return this.makeElement(name, arguments);
      });
      JS.DOM[name] = function() {
        return new JS.DOM.Builder().makeElement(name, arguments);
      };
    },
    
    addElements: function(list) {
      var i = list.length;
      while (i--) this.addElement(list[i]);
    }
  },
  
  initialize: function(parent) {
    this._parentNode = parent;
  },
  
  makeElement: function(name, children) {
    var element, child, attribute;
    if ( document.createElementNS ) {
      // That makes possible to mix HTML within SVG or XUL.
      element = document.createElementNS("http://www.w3.org/1999/xhtml", name);
    } else {
      element = document.createElement(name);
    }
    for (var i = 0, n = children.length; i < n; i++) {
      child = children[i];
      if (typeof child === 'function') {
        child(new this.klass(element));
      } else if (JS.isType(child, 'string')) {
        element.appendChild(document.createTextNode(child));
      } else {
        for (attribute in child)
          element[attribute] = child[attribute];
      }
    }
    if (this._parentNode) this._parentNode.appendChild(element);
    return element;
  },
  
  concat: function(text) {
    if (!this._parentNode) return;
    this._parentNode.appendChild(document.createTextNode(text));
  }
});

JS.DOM.Builder.addElements([
  "a", "abbr", "address", "applet", "area", "article", "aside", "audio", "b",
  "base", "bdo", "blockquote", "body", "br", "button", "canvas", "caption",
  "cite", "code", "col", "colgroup", "command", "datalist", "dd", "del",
  "details", "device", "dfn", "div", "dl", "dt", "em", "embed", "fieldset",
  "figcaption", "figure", "footer", "form", "h1", "h2", "h3", "h4", "h5", "h6",
  "head", "header", "hgroup", "hr", "html", "i", "iframe", "img", "input",
  "ins", "kbd", "keygen", "label", "legend", "li", "link", "map", "mark",
  "marquee", "menu", "meta", "meter", "nav", "noscript", "object", "ol",
  "optgroup", "option", "output", "p", "param", "pre", "progress", "q", "rp",
  "rt", "ruby", "samp", "script", "section", "select", "small", "source",
  "span", "strong", "style", "sub", "sup", "summary", "table", "tbody", "td",
  "textarea", "tfoot", "th", "thead", "time", "title", "tr", "track", "ul",
  "var", "video", "wbr"
]);


JS.DOM.Event = {
  _registry: [],
  
  on: function(element, eventName, callback, scope) {
    if (element !== JS.DOM.ENV &&
        element.nodeType !== JS.DOM.ELEMENT_NODE &&
        element.nodeType !== JS.DOM.DOCUMENT_NODE)
      return;
    
    var wrapped = function() { callback.call(scope, element) };
    
    if (element.addEventListener)
      element.addEventListener(eventName, wrapped, false);
    else if (element.attachEvent)
      element.attachEvent('on' + eventName, wrapped);
    
    this._registry.push({
      _element:   element,
      _type:      eventName,
      _callback:  callback,
      _scope:     scope,
      _handler:   wrapped
    });
  },
  
  detach: function(element, eventName, callback, scope) {
    var i = this._registry.length, register;
    while (i--) {
      register = this._registry[i];
      
      if ((element    && element    !== register._element)   ||
          (eventName  && eventName  !== register._type)      ||
          (callback   && callback   !== register._callback)  ||
          (scope      && scope      !== register._scope))
        continue;
      
      if (register._element.removeEventListener)
        register._element.removeEventListener(register._type, register._handler, false);
      else if (register._element.detachEvent)
        register._element.detachEvent('on' + register._type, register._handler);
      
      this._registry.splice(i,1);
      register = null;
    }
  }
};

JS.DOM.Event.on(JS.DOM.ENV, 'unload', JS.DOM.Event.detach, JS.DOM.Event);

==> jsclass-3.0.4.tgz: package/src/dom.js <==
JS.DOM = {
  ELEMENT_NODE:                   1,
  ATTRIBUTE_NODE:                 2,
  TEXT_NODE:                      3,
  CDATA_SECTION_NODE:             4,
  ENTITY_REFERENCE_NODE:          5,
  ENTITY_NODE:                    6,
  PROCESSING_INSTRUCTION_NODE:    7,
  COMMENT_NODE:                   8,
  DOCUMENT_NODE:                  9,
  DOCUMENT_TYPE_NODE:             10,
  DOCUMENT_FRAGMENT_NODE:         11,
  NOTATION_NODE:                  12,
  
  ENV: this,
  
  toggleClass: function(node, className) {
    if (this.hasClass(node, className)) this.removeClass(node, className);
    else this.addClass(node, className);
  },
  
  hasClass: function(node, className) {
    var classes = node.className.split(/\s+/);
    return JS.indexOf(classes, className) >= 0;
  },
  
  addClass: function(node, className) {
    if (this.hasClass(node, className)) return;
    node.className = node.className + ' ' + className;
  },
  
  removeClass: function(node, className) {
    var pattern = new RegExp('\\b' + className + '\\b\\s*', 'g');
    node.className = node.className.replace(pattern, '');
  }
};


JS.DOM.Builder = new JS.Class('DOM.Builder', {
  extend: {
    addElement: function(name) {
      this.define(name, function() {
        return this.makeElement(name, arguments);
      });
      JS.DOM[name] = function() {
        return new JS.DOM.Builder().makeElement(name, arguments);
      };
    },
    
    addElements: function(list) {
      var i = list.length;
      while (i--) this.addElement(list[i]);
    }
  },
  
  initialize: function(parent) {
    this._parentNode = parent;
  },
  
  makeElement: function(name, children) {
    var element, child, attribute;
    if ( document.createElementNS ) {
      // That makes possible to mix HTML within SVG or XUL.
      element = document.createElementNS("http://www.w3.org/1999/xhtml", name);
    } else {
      element = document.createElement(name);
    }
    for (var i = 0, n = children.length; i < n; i++) {
      child = children[i];
      if (typeof child === 'function') {
        child(new this.klass(element));
      } else if (JS.isType(child, 'string')) {
        element.appendChild(document.createTextNode(child));
      } else {
        for (attribute in child)
          element[attribute] = child[attribute];
      }
    }
    if (this._parentNode) this._parentNode.appendChild(element);
    return element;
  },
  
  concat: function(text) {
    if (!this._parentNode) return;
    this._parentNode.appendChild(document.createTextNode(text));
  }
});

JS.DOM.Builder.addElements([
  "a", "abbr", "address", "applet", "area", "article", "aside", "audio", "b",
  "base", "bdo", "blockquote", "body", "br", "button", "canvas", "caption",
  "cite", "code", "col", "colgroup", "command", "datalist", "dd", "del",
  "details", "device", "dfn", "div", "dl", "dt", "em", "embed", "fieldset",
  "figcaption", "figure", "footer", "form", "h1", "h2", "h3", "h4", "h5", "h6",
  "head", "header", "hgroup", "hr", "html", "i", "iframe", "img", "input",
  "ins", "kbd", "keygen", "label", "legend", "li", "link", "map", "mark",
  "marquee", "menu", "meta", "meter", "nav", "noscript", "object", "ol",
  "optgroup", "option", "output", "p", "param", "pre", "progress", "q", "rp",
  "rt", "ruby", "samp", "script", "section", "select", "small", "source",
  "span", "strong", "style", "sub", "sup", "summary", "table", "tbody", "td",
  "textarea", "tfoot", "th", "thead", "time", "title", "tr", "track", "ul",
  "var", "video", "wbr"
]);


JS.DOM.Event = {
  _registry: [],
  
  on: function(element, eventName, callback, context) {
    if (element !== JS.DOM.ENV &&
        element.nodeType !== JS.DOM.ELEMENT_NODE &&
        element.nodeType !== JS.DOM.DOCUMENT_NODE)
      return;
    
    var wrapped = function() { callback.call(context, element) };
    
    if (element.addEventListener)
      element.addEventListener(eventName, wrapped, false);
    else if (element.attachEvent)
      element.attachEvent('on' + eventName, wrapped);
    
    this._registry.push({
      _element:   element,
      _type:      eventName,
      _callback:  callback,
      _context:   context,
      _handler:   wrapped
    });
  },
  
  detach: function(element, eventName, callback, context) {
    var i = this._registry.length, register;
    while (i--) {
      register = this._registry[i];
      
      if ((element    && element    !== register._element)   ||
          (eventName  && eventName  !== register._type)      ||
          (callback   && callback   !== register._callback)  ||
          (context    && context    !== register._context))
        continue;
      
      if (register._element.removeEventListener)
        register._element.removeEventListener(register._type, register._handler, false);
      else if (register._element.detachEvent)
        register._element.detachEvent('on' + register._type, register._handler);
      
      this._registry.splice(i,1);
      register = null;
    }
  }
};

JS.DOM.Event.on(JS.DOM.ENV, 'unload', JS.DOM.Event.detach, JS.DOM.Event);
